<?php

namespace Drupal\layout_builder_enhancements_layout\Layouts;

use Drupal\Console\Core\Utils\NestedArray;
use Drupal\Core\Form\SubformState;
use Drupal\Core\Layout\LayoutDefault;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;

/**
 * Add layout settings.
 */
class FeatureLayout extends LayoutDefault implements PluginFormInterface {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {

    $featurePlugins = $this->getFeaturePlugins();
    if (!empty($featurePlugins)) {
      $form['plugins'] = [
        '#type' => 'container',
      ];
      $form['plugins']['settings_wrapper'] = [
        '#type' => 'container',
        '#id' => 'layout-feature-settings-wrapper',
        '#tree' => TRUE,
      ];
    }

    foreach ($featurePlugins as $key => $featurePlugin) {
      if ($featurePlugin instanceof PluginFormInterface) {
        $form['plugins']['settings_wrapper'][$key] = [
          '#type' => 'details',
          '#id' => 'layout-feature-settings-wrapper',
          '#title' => $featurePlugin->label(),
          '#description' => $featurePlugin->description(),
          '#open' => TRUE,
          '#tree' => TRUE,
        ];
        $subform_state = SubformState::createForSubform($form['plugins']['settings_wrapper'][$key], $form, $form_state);
        $form['plugins']['settings_wrapper'][$key] = $featurePlugin->buildConfigurationForm($form['plugins']['settings_wrapper'][$key], $subform_state);
      }
    }
    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $features = $this->getFeaturePlugins();

    foreach ($features as $key => $feature) {
      $subform_state = SubformState::createForSubform($form['plugins']['settings_wrapper'][$key] , $form, $form_state);
      $feature->submitConfigurationForm($form['plugins']['settings_wrapper'][$key], $subform_state);
      $this->configuration['features'][$key] = $feature->getConfiguration();
    }
  }

  /**
   * Get all feature plugins with configuration.
   *
   * @return array
   *   Features for this module.
   */
  protected function getFeaturePlugins(): array {
    $manager = \Drupal::service('plugin.manager.layout_feature_plugin');
    $definitions = $this->getPluginDefinition()->get('features');
    $features = [];
    foreach ($definitions as $key => $definition) {
      $features[$key] = $manager->createInstance($definition['plugin']['id'], $definition['plugin']);
      $configuration = [];
      if (isset($definition['configuration']) && is_array($definition['configuration'])) {
        $configuration = $definition['configuration'];
      }
      if (isset($this->configuration['features'][$key])) {
        $configuration = NestedArray::mergeDeepArray([$configuration, $this->configuration['features'][$key]], true);
      }
      $features[$key]->setConfiguration($configuration);
    }
    return $features;
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $regions) {
    $build = parent::build($regions);
    $build['#attributes']['class'] = $build['#attributes']['class'] ?? [];
    $featurePlugins = $this->getFeaturePlugins();
    foreach ($featurePlugins as $feature) {
     $feature->build($build, $regions);
    }
    return $build;
  }

}
