<?php

namespace Drupal\layout_builder_enhancements_layout\Plugin\LayoutFeature;

use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Drupal\image\Entity\ImageStyle;
use Drupal\layout_builder_enhancements_layout\Plugin\LayoutFeaturePluginBase;
use Symfony\Component\Validator\Constraints\ImageValidator;

/**
 * Add a background image.
 *
 * @LayoutFeaturePlugin(
 *   id = "background_image",
 *   label = "Background image",
 * )
 */
class BackgroundImage extends LayoutFeaturePluginBase
{

  /**
   * {@inheritDoc}
   */
  public function defaultConfiguration() {
    return [
      'default_location' => 'public://section-backgrounds/',
      'file_validate_extensions' => ['png gif jpg jpeg'],
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $configuration = $this->getConfiguration();
    $options = [];
    foreach ($configuration['classes'] as $key => $title) {
      $options[$key] = $title;
    }
    $form['image'] = [
      '#type' => 'managed_file',
      '#upload_location' => $configuration['default_location'],
      '#multiple' => FALSE,
      '#default_value' => (isset($configuration['image']) && is_array($configuration['image'])) ? $configuration['image'] : [],
      '#upload_validators' => [
        'file_validate_extensions' => $configuration['file_validate_extensions'],
      ],
    ];
    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritDoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->getValue('image')) {
      $this->setConfiguration([
        'image' => $form_state->getValue('image')
      ]);
    }
  }

  /**
   * {@inheritDoc}
   */
  public function build(array &$build, array $regions) {
    $configuration = $this->getConfiguration();
    $url = FALSE;
    if ($configuration['image']) {
      $file = File::load($configuration['image'][0]);

      if ($file) {
        $style = ImageStyle::load('large');
        $url = $style->buildUrl($file->getFileUri());
      }
    }
    if (!$url) {
      return;
    }
    if (!isset($build['#attributes']['style'])) {
      $build['#attributes']['style'] = '';
    }
    $build['#attributes']['style'] = 'background-image: url(' . $url . '); background-repeat: no-repeat; background-size: cover';
  }

}
