<?php

namespace Drupal\layout_builder_enhancements_layout\Plugin;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\Component\Plugin\ConfigurableInterface;
use Drupal\Component\Utility\NestedArray;

/**
 * Base class for Layout feature plugins.
 */
abstract class LayoutFeaturePluginBase extends PluginBase implements LayoutFeaturePluginInterface, PluginFormInterface, ConfigurableInterface {

  /**
   * Prepare configuration.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Formstate interface.
   * @param array $form
   *   Current form.
   *
   * @return array
   *   Return configuration.
   */
  abstract public function build(array &$build, array $regions);

  /**
   * {@inheritDoc}
   */
  public function getConfiguration() {
    return NestedArray::mergeDeepArray([$this->defaultConfiguration(), $this->configuration], true);
  }

  /**
   * {@inheritDoc}
   */
  public function setConfiguration(array $configuration) {
    $this->configuration = $configuration;
  }

  /**
   * {@inheritDoc}
   */
  public function defaultConfiguration() {
    return $this->pluginDefinition['configuration'];
  }

  /**
   * Get label.
   *
   * @return string
   *   Defined label.
   */
  public function label() {
    return $this->pluginDefinition['label'];
  }

  /**
   * Get description.
   *
   * @return string
   *   Defined label.
   */
  public function description() {
    if (isset($this->pluginDefinition['description'])) {
      return $this->pluginDefinition['description'];
    }
    return '';
  }

}
